#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
import gtk, gobject
from gui import pixbufLoader
from gui.itemStatsWindow import itemStatsWindow
from model import fitting, item, group
import copy

def init(self):
    #Register our custom events and their handlers
    gobject.signal_new("fitting-changed", self, gobject.SIGNAL_RUN_FIRST,
                       gobject.TYPE_NONE, ())
    self.connect("fitting-changed", self.fittingChanged)
    self.tvwModules.connect("button-press-event", self.tvwModulesClicked)
    self.tvwModules.connect("key-press-event", self.tvwModulesKeyPress)
    #itemIcon, Name, description, PG, cpu, cap, range, ammoIcon, ammo, item
    #Get needed images
    pgImage = gtk.image_new_from_pixbuf(pixbufLoader.getPixbuf("pgSmall"))
    cpuImage = gtk.image_new_from_pixbuf(pixbufLoader.getPixbuf("cpuSmall"))
    capImage = gtk.image_new_from_pixbuf(pixbufLoader.getPixbuf("capacitorSmall"))
    rangeImage = gtk.image_new_from_pixbuf(pixbufLoader.getPixbuf("range"))

    #Call show on all the images, they're very shy
    pgImage.show()
    cpuImage.show()
    capImage.show()
    rangeImage.show()

    #Make and set store
    store = gtk.ListStore(gtk.gdk.Pixbuf, str, str, str, str, str, str, gtk.gdk.Pixbuf, str, object, object, str, gtk.gdk.Pixbuf, int, str, bool)
    self.tvwModules.set_model(store)

    #Setup sorting
    store.set_sort_func(11, self.sortBySlot)
    store.set_sort_column_id(11, gtk.SORT_ASCENDING)

    #Cell renderers are our good old friends LETS MAKE LOTS OF THEM
    itemStateRenderer = gtk.CellRendererPixbuf()
    itemIconRenderer = gtk.CellRendererPixbuf()
    nameRenderer = gtk.CellRendererText()
    descriptionRenderer = gtk.CellRendererText()
    pgRenderer = gtk.CellRendererText()
    cpuRenderer = gtk.CellRendererText()
    capRenderer = gtk.CellRendererText()
    rangeRenderer = gtk.CellRendererText()
    ammoIconRenderer = gtk.CellRendererPixbuf()
    ammoRenderer = gtk.CellRendererText()

    #tree view columns are also our friends LETS INVITE A FEW OVER
    self.itemStateCol = gtk.TreeViewColumn()
    nameCol = gtk.TreeViewColumn("Name")
    pgCol = gtk.TreeViewColumn()
    pgCol.set_widget(pgImage)
    cpuCol = gtk.TreeViewColumn()
    cpuCol.set_widget(cpuImage)
    capCol = gtk.TreeViewColumn()
    capCol.set_widget(capImage)
    rangeCol = gtk.TreeViewColumn()
    rangeCol.set_widget(rangeImage)
    ammoCol = gtk.TreeViewColumn("Ammo")

    #Cell renderers, meet columns. Columns, meet cell renderers
    self.itemStateCol.pack_start(itemStateRenderer, False)
    nameCol.pack_start(itemIconRenderer, False)
    nameCol.pack_start(nameRenderer, True)
    pgCol.pack_start(pgRenderer, False)
    cpuCol.pack_start(cpuRenderer, False)
    capCol.pack_start(capRenderer, False)
    rangeCol.pack_start(rangeRenderer, False)
    ammoCol.pack_start(ammoIconRenderer, False)
    ammoCol.pack_start(ammoRenderer, False)

    #Ok, here's your info, NOW LEAVE ME ALONE
    self.itemStateCol.add_attribute(itemStateRenderer, 'pixbuf', 12)
    nameCol.add_attribute(itemIconRenderer, 'pixbuf', 0)
    nameCol.add_attribute(nameRenderer, 'text', 1)
    nameCol.add_attribute(nameRenderer, "foreground-set", 15)
    nameRenderer.set_property("foreground", gtk.gdk.color_parse('red'))
    pgCol.add_attribute(pgRenderer, 'text', 3)
    cpuCol.add_attribute(cpuRenderer, 'text', 4)
    capCol.add_attribute(capRenderer, 'text', 5)
    rangeCol.add_attribute(rangeRenderer, 'text', 6)
    ammoCol.add_attribute(ammoIconRenderer, 'pixbuf', 7)
    ammoCol.add_attribute(ammoRenderer, 'text', 8)

    #Columns, meet treeview. Treeview, meet columns
    tv = self.tvwModules
    tv.append_column(self.itemStateCol)
    tv.append_column(nameCol)
    tv.append_column(pgCol)
    tv.append_column(cpuCol)
    tv.append_column(capCol)
    tv.append_column(rangeCol)
    tv.append_column(ammoCol)

    #Set tooltip with description
    tv.set_tooltip_column(2)

    #Tooltip stuff for lock times
    w = gtk.Window(gtk.WINDOW_POPUP)
    f = gtk.Frame()
    f.set_border_width(1)
    w.set_decorated(False)
    self.lockTimesTable = gtk.Table(1, 2)
    lbl = gtk.Label()
    lbl.set_markup("<b>Lock Times</b>")
    self.lockTimesTable.attach(lbl, 0, 3, 0, 1)
    self.lockTimeLabels = []
    f.add(self.lockTimesTable)
    w.add(f)
    w.set_name('gtk-tooltips')
    f.show_all()

    for widget in ("lblScanResHeader", "boxScanRes"):
        widget = getattr(self, widget)
        widget.set_property("has-tooltip", True)
        widget.set_tooltip_window(w)
        widget.connect("query-tooltip", lambda one, two, three, four, five: self.getActiveFit() != None)

    #Generate the menus and menuitems we need
    self.fittedModuleMenu = self.generateMenu({gtk.Label("Ship Stats") : self.fittedShipStats,
                                               gtk.Label("Add to favorites") : self.addModuleFavorite})
    self.fittedModuleStatsEntry = self.generateMenuItem(gtk.Label("Item Stats"), self.fittedModuleStats)
    self.fittedModuleGroupEntry = self.generateMenuItem(gtk.Label("Item Market Group"), self.fittedModuleGroup)
    self.fittedAmmoStatsEntry = self.generateMenuItem(gtk.Label("Charge Stats"), self.fittedAmmoStats)
    self.fittedAmmoGroupEntry = self.generateMenuItem(gtk.Label("Charge Market Group"), self.fittedAmmoGroup)
    self.fittedAmmoChangeEntry = self.generateMenuItem(gtk.Label("Change Charge"), None)

    #Set tooltips for other stuff
    self.setTooltips(**{"boxTurretSlots" : "Turret hardpoints",
                        "boxLauncherSlots" : "Launcher hardpoints",
                        "boxCalibration" : "Calibration",
                        "boxCpu" :  "CPU",
                        "boxPower" : "Powergrid",
                        "boxDroneBay" : "Drone bay volume",
                        "boxDroneBandwidth" : "Drone control bandwidth",
                        "imgShield" : "Shield",
                        "imgArmor" : "Armor",
                        "imgHull" : "Hull",
                        "imgEm" : "EM resistance",
                        "imgThermal" : "Thermal resistance",
                        "imgKinetic" : "Kinetic resistance",
                        "imgExplosive" : "Explosive resistance",
                        "lblEhpHeader" : "Effective HP",
                        "imgPassiveShield" : "Shield regeneration peak",
                        "imgActiveShield" : "Active shield tank",
                        "imgActiveArmor" : "Armor tank",
                        "imgActiveHull" : "Hull tank",
                        "imgReinforced" : "Reinforced tank",
                        "imgSustained" : "Sustained tank",
                        "boxShipDps" : "Ship weapon DPS",
                        "boxDroneDps" : "Drone DPS",
                        "lblVolleyHeader" : "Volley damage",
                        "lblVolley" : "Volley damage"})

    self.tvwModules.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
    self.tvwModules.set_rubber_banding(True)
    self.tvwModules.set_reorderable(True)

    self.tvwModules.enable_model_drag_dest(
        [('text/plain', 0, 0)],
        gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE
    )
    self.tvwModules.enable_model_drag_source(
        gtk.gdk.BUTTON1_MASK, 
        [('text/plain', 0, 0)],
        gtk.gdk.ACTION_DEFAULT | gtk.gdk.ACTION_MOVE
    )
    self.tvwModules.connect("drag-data-received", self.dragDataRecv)
    self.tvwModules.connect("drag-data-get", self.dragDataSend)

def dragDataRecv(self, widget, drag_context, x, y, selection_data, info, timestamp):
    #Figure where to move to
    drop_info = widget.get_dest_row_at_pos(x, y)
    if drop_info == None or drop_info[0] == None: return
    #Change dest depending on where exactly we need to move
    dest, info = drop_info
    #Figure where the items to move are at
    paths = self.tvwModules.get_selection().get_selected_rows()[1]
    if not paths or dest in paths: return
    if info == gtk.TREE_VIEW_DROP_AFTER: dest = (dest[0] + 1,)
    store = self.tvwModules.get_model()
    #print "destination", dest
    destIter = store.get_iter(dest)
    #Figure what's at the destination
    destItem, destAmmo, destSlot, destIndex = store.get(destIter, 9, 10, 11, 13)
    activeFit = self.getActiveFit()
    #Put indice in a list
    sourceIndice = {}
    for i in range(len(activeFit.modules)): sourceIndice[i] = i
    for path in paths:
        #Figure where we need to drop exactly
        iter = store.get_iter(path)
        srcIndex = store.get_value(iter, 13)
        srcItem, srcState, srcAmmo = activeFit.modules[srcIndex]
        srcSlot = srcItem.getModifiedAttribute("_slot")
        if srcSlot != destSlot:
            refactor = True
            #Slots aren't of the same types
            #figure if a slot type with higher or lower prio was selected
            before = fitting.slotTypes.index(srcSlot) <= fitting.slotTypes.index(destSlot)
            #Figure the valid index of the same slot to add the item at
            i = before and len(activeFit.modules) - 1 or 0
            while before and i > 0 or i < 0:
                i = before and i - 1 or i + 1
                if activeFit.modules[i][0].getModifiedAttribute("_slot") == srcSlot:
                    destIndex = i
                    break
        else: refactor = False
        #Figure where sourceItem moved to (if it moved)
        srcIndex = sourceIndice[srcIndex]
        del activeFit.modules[srcIndex]
        for orig, current in sourceIndice.iteritems():
            if (info == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE or\
                info == gtk.TREE_VIEW_DROP_INTO_OR_AFTER or \
                refactor == True) and \
            current == destIndex: continue
            if current > srcIndex: sourceIndice[orig] = current - 1
        
            
        #Figure where the destIndex moved to
        destIndex = sourceIndice[destIndex]
        #Insert it at the correct location now
        activeFit.modules.insert(destIndex, (srcItem, srcState, srcAmmo))
        #Change the indice dict accordingly again
        for orig, current in sourceIndice.iteritems():
            if current >= destIndex: sourceIndice[orig] = current + 1
            
    self.emit("fitting-changed")

def addModuleFavorite(self, *stuff):
    self.favoriteItems.add(self.getSelectedModule()[0].ID)
    self.groupSelected(self.tvwGroups)
    
def getActiveFit(self):
    store = self.cmbSetup.get_model()
    if store == None: return
    iter = self.cmbSetup.get_active_iter()
    if iter == None: return
    return store.get(iter, 2)[0]

def generateAmmoMenu(self, modules, callback):
    ammoTypes = {}
    metaOrder = ("normal", "faction", "deadspace", "officer")
    chargeGroups = set()
    for mod in modules:
        for i in range(5):
            chargeGroup = mod.getModifiedAttribute('chargeGroup%d' % i)
            if chargeGroup != None: chargeGroups.add(chargeGroup)

    for chargeGroup in chargeGroups:
        grp = group.getGroup(ID = chargeGroup)
        items = item.getItemsByGroup(group=grp)
        itemsToAdd = {}
        for it in items:
            if it.published == 1:
                skip = False
                for mod in modules:
                    if not fitting.ammoFits(mod, it): skip = True
                if skip: continue
                if it.metaGroupID == 4: meta = "faction"
                elif it.metaGroupID == 5: meta = "officer"
                elif it.metaGroupID == 6: meta = "deadspace"
                else: meta = "normal"
                if meta not in itemsToAdd: itemsToAdd[meta] = []
                itemsToAdd[meta].append(it)

        if len(itemsToAdd) > 0:
            ammoTypes[grp.name] = itemsToAdd

    if len(ammoTypes) > 0:
        menu = gtk.Menu()
        for grpName, metas in ammoTypes.iteritems():
            if grp.icon: pixbuf = pixbufLoader.getPixbuf(grp.icon, True, "16x16")
            else: pixbuf = None
            box = gtk.HBox()
            if pixbuf: box.pack_start(gtk.image_new_from_pixbuf(pixbuf), False)
            lbl = gtk.Label(grpName)
            lbl.set_alignment(0, 0.5)
            box.pack_start(lbl, False)
            grpMenu = gtk.Menu()
            grpMenuItem = gtk.MenuItem()
            grpMenuItem.add(box)
            grpMenuItem.set_submenu(grpMenu)
            menu.append(grpMenuItem)
            for meta in metaOrder:
                if meta in metas:
                    items = metas[meta]
                    if len(metas) > 1:
                        submenu = gtk.Menu()
                        menuItem = gtk.MenuItem()
                        lbl = gtk.Label(meta.capitalize())
                        lbl.set_alignment(0, 0.5)
                        menuItem.add(lbl)
                        menuItem.set_submenu(submenu)
                        grpMenu.append(menuItem)
                    else: submenu = grpMenu
                    items.sort(cmp = lambda one, two: cmp(one.name, two.name))
                    for it in items:
                        if it.icon: pixbuf = pixbufLoader.getPixbuf(it.icon, True, "16x16")
                        else: pixbuf = None
                        box = gtk.HBox()
                        if pixbuf: box.pack_start(gtk.image_new_from_pixbuf(pixbuf), False)
                        lbl = gtk.Label(it.name)
                        lbl.set_alignment(0, 0.5)
                        box.pack_start(lbl, False)
                        itMI = gtk.MenuItem()
                        if callback: itMI.connect("activate", callback, it)
                        itMI.add(box)
                        submenu.append(itMI)
        
        lbl = gtk.Label("None")
        lbl.set_alignment(0, 0.5)
        mi = gtk.MenuItem()
        mi.add(lbl)
        mi.connect("activate", callback, None)
        menu.append(mi)
    else: menu = None
    return menu

def generateFitMenu(self, callback, parent = None):
    store = self.cmbSetup.get_model()
    iter = store.iter_children(parent)
    if iter == None: return
    menu = gtk.Menu()
    while iter:
        menuItem = gtk.MenuItem()
        sub = self.generateFitMenu(callback, iter)
        if sub:
            pixbuf, name = store.get(iter, 0, 1)
            menuItem.set_submenu(sub)
        else:
            pixbuf = None
            name, fit = store.get(iter, 1, 2)
            menuItem.connect("activate", callback, fit)
        box = gtk.HBox()
        box.pack_start(gtk.image_new_from_pixbuf(pixbuf), False)
        lbl = gtk.Label(name)
        lbl.set_alignment(0.0, 0.5)
        box.pack_start(lbl, True)
        menuItem.add(box)
        menuItem.show_all()
        menu.append(menuItem)
        iter = store.iter_next(iter)

    return menu

def getSelectedModule(self):
    store = self.tvwModules.get_model()
    select = self.tvwModules.get_cursor()[0]
    iter = store.get_iter(select)
    return store.get(iter, 9, 10)

def fittedShipStats(self, *stuff):
    fit = self.getActiveFit()
    itemStatsWindow(fit.ship, fit, pixbufLoader.getPixbuf(fit.ship.race))

def fittedAmmoStats(self, *stuff):
    itemStatsWindow(self.getSelectedModule()[1], self.getActiveFit(), None)

def fittedAmmoGroup(self, *stuff): self.expandToItem(self.getSelectedModule()[1])

def fittedModuleStats(self, *stuff):
    store = self.tvwModules.get_model()
    rows = self.tvwModules.get_selection().get_selected_rows()[1]
    modules = set()
    for row in rows:
        iter = store.get_iter(row)
        module, ammo = store.get(iter, 9, 10)
        if module != None: modules.add((module, ammo))

    for mod, ammo in modules:
        itemStatsWindow(module, self.getActiveFit(), None, ammo)

def fittedModuleGroup(self, *stuff): self.expandToItem(self.getSelectedModule()[0])

def fittingChanged(self, *stuff):
    startLoc = self.tvwModules.get_cursor()[0]
    moduleStore, selected_rows = self.tvwModules.get_selection().get_selected_rows()
    if moduleStore == None: return True
    moduleStore.clear()

    #Figure the character
    char = self.getActiveCharacter()
    #Save the character
    if char != None:
        self.guiSettings["activeCharacter"] = char.name

    fit = self.getActiveFit()
    if fit == None:
        labels = ("lblTurretSlotsUsed", "lblTurretSlotsMax", "lblLauncherSlotsUsed",
                  "lblLauncherSlotsMax", "lblCalibrationUsed", "lblCalibrationMax",
                  "lblCpuUsed", "lblCpuMax", "lblPowerUsed", "lblPowerMax",
                  "lblDroneBayUsed", "lblDroneBayMax", "lblDroneBandwidthUsed",
                  "lblDroneBandwidthMax", "lblPassiveShieldTank", "lblShieldTank",
                  "lblArmorTank", "lblHullTank", "lblCapCapacity", "lblCapState",
                  "lblSustainedArmorTank", "lblSustainedHullTank", "lblEhp",
                  "lblSustainedShieldTank", "lblCapRecharge", "lblCapDischarge",
                  "lblShipDps", "lblDroneDps", "lblVolley", "lblTargets", "lblRange",
                  "lblScanRes", "lblScanStr", "lblSpeed", "lblAlignTime", "lblCargo",
                  "lblSigRadius", "lblDronesUsed", "lblDronesTotal", "lblTotalDps")
        for lbl in labels:
            getattr(self, lbl).set_text("0")
        for type in ("Shield", "Armor", "Hull"):
            getattr(self, "lbl" + type + "Ehp").set_text("0")
            for dmg in ("Em", "Thermal", "Kinetic", "Explosive"):
                getattr(self, "lbl" + type + dmg + "Resistance").set_text("0.0")
        for pbar in ("Cpu", "Power", "DroneBay", "DroneBandwidth"):
            bar = getattr(self, "pbar" + pbar)
            bar.set_text("")
            bar.set_fraction(0.0)

        return True

    fit.character = char
    #Start of missing skills stuff

    fit.calculateModifiedAttributes()
    #Check if any mods need disabling
    if(fit.disableImpossibleMods()):
        self.emit("fitting-changed")
        return True
    
    #Remove superfluous dummies/Add missing ones        
    for slotType in fitting.slotTypes:
        left = fit.getSlotsLeft(slotType)
        numDummies = 0
        i = 0
        for item, state, ammo in fit.modules:
            if item.ID == 0 and item.getModifiedAttribute("_slot") == slotType: numDummies += 1
        left = left - numDummies
        for i in range(int(left)):
            fit.modules.append((fitting.generateDummy(slotType), None, None))


    fit.calculateCapUsage()
    #Add modules to the view
    red = set()
    for i in range(len(fit.modules)):
        item, state, ammo = fit.modules[i]
        #This item is a dummy
        if item.ID == 0:
            pixbufItem = pixbufLoader.getPixbuf(item.getModifiedAttribute("_slot"))
        else:
            pixbufItem = pixbufLoader.getPixbuf(item.icon, True, "16x16")
        
            
        if state is not None:
            pixbufState = pixbufLoader.getPixbuf(fitting.modStates[state])
        else:
            pixbufState = None
        if ammo != None:
            pixbufAmmo = pixbufLoader.getPixbuf(ammo.icon, True, "16x16")
            nameAmmo = ammo.name
        else:
            pixbufAmmo = None
            nameAmmo = None

        power = item.getModifiedAttribute("power") or 0
        cpu = item.getModifiedAttribute("cpu") or 0
        if int(power) == power:
            power = "%d" % power
        else:
            power = '%.1f' % power
        if int(cpu) == cpu:
            cpu = "%d" % cpu
        else:
            cpu = '%.1f' % cpu


        capUsage = item.getModifiedAttribute("_capUsage")
        capBoost = item.getModifiedAttribute("_capBoost")
        if capUsage:
            cap = "%.1f" % capUsage
        elif capBoost:
            cap = "+%.1f" % capBoost
        else:
            cap = ""

        maxRange = item.getModifiedAttribute("maxRange") or item.getModifiedAttribute("_maxRange")
        falloff = item.getModifiedAttribute("falloff")
        if maxRange != None:
            maxRange = maxRange / 1000.0
            if falloff != None and falloff != 0:
                falloff = falloff / 1000.0
                rangeStr = '%.1f + %.1fkm' % (maxRange, falloff)
            else:
                missile = False
                for skill in item.requiredSkills:
                    if skill.name == "Missile Launcher Operation":
                        missile = True
                        break
                rangeStr = '%s%.1fkm' % (missile and "<" or "", maxRange)

        else:
            rangeStr = None
        moduleStore.append((pixbufItem, item.name, item.description, power, cpu, cap, rangeStr, pixbufAmmo, nameAmmo, item, ammo, item.attributes["_slot"].getModifiedValue(), pixbufState, i, None, None))

    #Now we're done set the cursor back to where it was
    if startLoc != None and startLoc != -1:
        self.tvwModules.set_cursor(startLoc)
    for row in selected_rows:
        self.tvwModules.get_selection().select_path(row)

    #Right pane
    #Turret and launcher slots
    for slotType in ("turret", "launcher"):
        used = fit.getNumModulesAttributeValue("_hardpoint", slotType)
        try: total = fit.ship.getModifiedAttribute(slotType + "SlotsLeft") + used
        except Exception: total = 0
        lblUsed = getattr(self, "lbl" + slotType.capitalize() + "SlotsUsed")
        if used > total:
            lblUsed.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('red'))
        else:
            lblUsed.modify_fg(gtk.STATE_NORMAL, None)

        lblUsed.set_text(str(used))
        getattr(self, "lbl" + slotType.capitalize() + "SlotsMax").set_text('%d' % total)

    #Calibration
    used = fit.getModuleAttributeSum("upgradeCost")
    total = fit.ship.attributes["upgradeCapacity"].getModifiedValue()
    if used > total:
        self.lblCalibrationUsed.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('red'))
    else:
        self.lblCalibrationUsed.modify_fg(gtk.STATE_NORMAL, None)

    self.lblCalibrationUsed.set_text(str(int(used)))
    self.lblCalibrationMax.set_text(str(int(total)))

    #cpu and power
    for type in ("cpu", "power"):
        used = fit.getModuleAttributeSum(type, fitting.STATE_INACTIVE)
        total = fit.ship.attributes[type + "Output"].getModifiedValue()
        type = type.capitalize()
        lblUsed = getattr(self, "lbl" + type + "Used")
        lblUsed.set_text('%.1f' % used)
        getattr(self, "lbl" + type + "Max").set_text('%.1f' % total)
        if total == 0:
            if used > 0:
                fraction = float("inf")
            else:
                fraction = 0.0
        else:
            fraction = used * 1.0 / total
        pbar = getattr(self, "pbar" + type)
        if fraction > 1.0:
            if fraction == float("inf"):
                self.pbarDroneBay.set_text(u"+\u221E")
            else:
                excess = fraction - 1
                pbar.set_text('+%.2f%%' % (excess * 100))
            #pbar.modify_fg(gtk.STATE_PRELIGHT, gtk.gdk.color_parse('red'))
            lblUsed.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('red'))
        else:

            pbar.set_text('%.2f%%' % (fraction * 100))
            #pbar.modify_fg(gtk.STATE_PRELIGHT, None)
            lblUsed.modify_fg(gtk.STATE_NORMAL, None)

        pbar.set_fraction(min(fraction, 1.0))

    #Drone bay
    used = fit.getDronesAttributeSum("volume", False)
    total = fit.ship.attributes["droneCapacity"].getModifiedValue()
    self.lblDroneBayUsed.set_text('%.0f' % used)
    self.lblDroneBayMax.set_text('%.0f' % total)
    if total != 0:
        fraction = used * 1.0 / total
    else:
        if used == 0:
            fraction = 0
        else:
            fraction = float("inf")

    if fraction > 1.0:
        if fraction == float("inf"):
            self.pbarDroneBay.set_text(u"+\u221E")
        else:
            excess = fraction - 1
            self.pbarDroneBay.set_text('+%.2f%%' % (excess * 100))

        self.lblDroneBayUsed.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('red'))
        #self.pbarDroneBay.modify_fg(gtk.STATE_PRELIGHT, gtk.gdk.color_parse('red'))
    else:
        self.pbarDroneBay.set_text('%.2f%%' % (fraction * 100))
        self.lblDroneBayUsed.modify_fg(gtk.STATE_NORMAL, None)
        #self.pbarDroneBay.modify_fg(gtk.STATE_NORMAL, None)

    self.pbarDroneBay.set_fraction(min(fraction, 1.0))

    #Drone bandwidth
    used = fit.getDronesAttributeSum("droneBandwidthUsed", True)
    total = fit.ship.attributes["droneBandwidth"].getModifiedValue()
    self.lblDroneBandwidthUsed.set_text('%.0f' % used)
    self.lblDroneBandwidthMax.set_text('%.0f' % total)
    if total != 0:
        fraction = used * 1.0 / total
    else:
        if used == 0:
            fraction = 0
        else:
            fraction = float("inf")

    if fraction > 1.0:
        if fraction == float("inf"):
            self.pbarDroneBandwidth.set_text(u"+\u221E")
        else:
            excess = fraction - 1
            self.pbarDroneBandwidth.set_text('+%.2f%%' % (excess * 100))

        self.lblDroneBandwidthUsed.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse('red'))
        #self.pbarDroneBandwidth.modify_fg(gtk.STATE_PRELIGHT, gtk.gdk.color_parse('red'))
    else:
        self.pbarDroneBandwidth.set_text('%.2f%%' % (fraction * 100))
        self.lblDroneBandwidthUsed.modify_fg(gtk.STATE_NORMAL, None)
        #self.pbarDroneBandwidth.modify_fg(gtk.STATE_PRELIGHT, None)
    self.pbarDroneBandwidth.set_fraction(min(fraction, 1.0))

    #capacitor
    drain = fit.ship.getModifiedAttribute("_drain") or 0
    capacity = fit.ship.getModifiedAttribute("capacitorCapacity")
    recharge = fit.ship.getModifiedAttribute("_peakCapRecharge")
    discharge = fit.getModuleAttributeSum("_capUsage") + drain
    stable = fit.ship.getModifiedAttribute("_capStable")
    if stable == True:
        capStatePrefix = "Stable at "
        capState = '%.1f' % fit.ship.getModifiedAttribute("_capState")
        capStateSuffix = " %"
    else:
        capStatePrefix = "Lasts "
        capState = fit.ship.getModifiedAttribute("_capState")
        if capState >= 100:
            capState = '%.0fm%.0f' % ((capState / 60) , (capState % 60))
        else:
            capState = '%.0f' % capState

        capStateSuffix = "s"

    self.lblCapCapacity.set_text('%.1f' % capacity)
    self.lblCapRecharge.set_text('%.1f' % recharge)
    self.lblCapDischarge.set_text('%.1f' % -discharge)
    self.lblCapStatePrefix.set_text(capStatePrefix)
    self.lblCapState.set_text(capState)
    self.lblCapStateSuffix.set_text(capStateSuffix)

    #Ship DPS and volley
    fit.calculateVolley()
    fit.calculatePerSecond("_volley", "_dps")
    fit.calculatePerSecond("miningAmount", "_miningYield")
    miningYield = fit.getModuleAttributeSum("_miningYield")
    shipDps = fit.getModuleAttributeSum("_dps")
    if shipDps or not miningYield:
        volley = fit.getModuleAttributeSum("_volley")
        droneDps = fit.getDronesAttributeSum("_dps", True)
        self.lblFirePowerHeader.set_markup("<b>Firepower</b>")
        self.imgShipDps.set_from_pixbuf(pixbufLoader.getPixbuf("turret"))
        self.lblVolleyHeader.set_text("Volley: ")
        for lbl in ("lblShipDpsUnit", "lblDroneDpsUnit"):
            getattr(self, lbl).set_text(" DPS")
        self.lblTotalDpsHeader.set_text("Total DPS: ")
    else: #Switch over to mining yield
        shipDps = miningYield
        volley = fit.getModuleAttributeSum("miningAmount")
        droneDps = fit.getDronesAttributeSum("_miningYield", True)
        self.lblFirePowerHeader.set_markup("<b>Mining Yield</b>")
        self.imgShipDps.set_from_pixbuf(pixbufLoader.getPixbuf("mining"))
        self.lblVolleyHeader.set_text("Yield: ")
        for lbl in ("lblShipDpsUnit", "lblDroneDpsUnit"):
            getattr(self, lbl).set_text(" MPS")
        self.lblTotalDpsHeader.set_text("Total MPS: ")

    self.lblShipDps.set_text('%.1f' % shipDps)
    self.lblVolley.set_text('%.1f' % volley)

    #Drone DPS
    self.lblDroneDps.set_text('%.1f' % droneDps)

    #total dps
    self.lblTotalDps.set_text('%.1f' % (droneDps + shipDps))

    #Effective hp and resistance
    if self.activePattern == "raw": damagePattern = "raw"
    else: damagePattern = self.damagePatterns[self.activePattern]
    fit.calculateTankEhp(damagePattern)
    for (layer, attrPrefix) in (("Shield", "shield"), ("Armor", "armor"), ("Hull", "")):
        for damageType in ("Kinetic", "Thermal", "Explosive", "Em"):
            lbl = getattr(self, "lbl" + layer + damageType + "Resistance")
            attrName = attrPrefix + damageType + "DamageResonance"
            attrName = attrName[0].lower() + attrName[1:]
            resonance = fit.ship.getModifiedAttribute(attrName)
            resistance = (1 - (resonance or 1)) * 100
            lbl.set_text('%.2f' % resistance)

        ehp = fit.ship.getModifiedAttribute("_" + attrPrefix.capitalize() + "Ehp")
        getattr(self, "lbl" + layer + "Ehp").set_text('%.0f' % ehp)

    self.lblEhp.set_text('%d' % fit.ship.getModifiedAttribute("_totalEhp"))

    #Tank
    fit.calculateTankRegen(damagePattern)
    fit.calculateSustainedTankRegen()
    for tankType in ("passiveShield", "shield", "armor", "hull"):
        for subType in ("", "Sustained"):
            if subType == "Sustained" and tankType == "passiveShield": continue
            lblName = "lbl" + subType + tankType[0].upper() + tankType[1:] + "Tank"
            lbl = getattr(self, lblName)
            attrName = "_" + tankType + subType + "Regen"
            tank = fit.ship.getModifiedAttribute(attrName) or 0
            lbl.set_text('%.1f' % tank)

    #Targetting related info
    #Targetting range
    targetRange = fit.ship.getModifiedAttribute("maxTargetRange") / 1000
    self.lblRange.set_text("%.1f" % targetRange)

    #Max locked targets
    maxTargets = min(fit.ship.getModifiedAttribute("_maxLockedTargetsSkill"),
                     fit.ship.getModifiedAttribute("maxLockedTargets"))
    self.lblTargets.set_text('%.0f' % maxTargets)

    #scan Resolution
    scanResolution = fit.ship.getModifiedAttribute("scanResolution")
    self.lblScanRes.set_text('%.1f' % scanResolution)

    #Lock times tooltip
    signatures = [("Interceptor", 32),
                  ("Frigate", 46),
                  ("Cruiser", 110),
                  ("Battlecruiser", 272),
                  ("Battleship", 470)]

    self.lockTimesTable.set_property("n-rows", len(signatures) + 1)
    for lbl in self.lockTimeLabels: self.lockTimesTable.remove(lbl)
    del self.lockTimeLabels[:]
    for i in range(len(signatures)):
        shipType, shipSig = signatures[i]
        lockTime = fit.calculateLockTime(shipSig)
        lblShip = gtk.Label()
        self.lockTimeLabels.append(lblShip)
        lblShip.set_markup("%s [%d]: " % (shipType, shipSig))
        lblShip.set_alignment(1.0, 0.5)
        self.lockTimesTable.attach(lblShip, 0, 1, i + 1, i + 2)
        lblTime = gtk.Label("%.1fs" % lockTime)
        self.lockTimeLabels.append(lblTime)
        self.lockTimesTable.attach(lblTime, 2, 3, i + 1, i + 2)

    self.lockTimesTable.show_all()


    #Scan strength, Pick the highest one to display
    scanStr = 0
    for scanType in ("Magnetometric", "Ladar", "Radar", "Gravimetric"):
        currScanStr = fit.ship.getModifiedAttribute("scan" + scanType + "Strength")
        if currScanStr > scanStr:
            scanStr = currScanStr

    self.lblScanStr.set_text('%.0f' % scanStr)

    #Speed
    speed = fit.ship.getModifiedAttribute("maxVelocity")
    if speed > 1000:
        speed = speed / 1000.0
        self.lblSpeedUnit.set_text(" km/s")
        self.lblSpeed.set_text('%.3f' % speed)
    else:
        self.lblSpeedUnit.set_text(" m/s")
        self.lblSpeed.set_text('%.1f' % speed)

    fit.calculateAlignTime()
    #Align time
    alignTime = fit.ship.getModifiedAttribute("_alignTime")
    self.lblAlignTime.set_text("%.1f" % alignTime)

    #Cargo hold
    cargo = fit.ship.getModifiedAttribute("capacity")
    if cargo > 1000:
        cargo = cargo / 1000
        self.lblCargoUnit.set_markup(" k m\302\263")
    else:
        self.lblCargoUnit.set_markup(" m\302\263")

    self.lblCargo.set_text('%.1f' % cargo)

    #Signature radius
    sigRadius = fit.ship.getModifiedAttribute("signatureRadius")
    self.lblSigRadius.set_text('%.0f' % sigRadius)

    #Set the reload time togglebutton correctly
    self.includeReloadTimeToggle.set_active(fit.factorInReload)

def manageRecent(self, item):
    if item.ID in self.recentItems: self.recentItems.remove(item.ID)
    self.recentItems.append(item.ID)
    while len(self.recentItems) > 30: self.recentItems.pop(0)
    grp = self.getSelectedGroup()
    if grp == "recent" and not self.itemSearchActive: self.groupSelected(self.tvwGroups)
    
def addItem(self, treeview):
    #Get item being added
    itemStore = treeview.get_model()
    path = treeview.get_cursor()[0]
    if path == -1 or path == None:
        return

    iter = itemStore.get_iter(path)
    it = itemStore.get(iter, 3)[0]
    if not isinstance(it, item.item):
        return
    #Get item with all it's info
    it = item.getItem(ID = it.ID)
    #Get current fit
    fit = self.getActiveFit()
    if fit == None: return
    cat = it.group.category.name
    if cat == "Drone":
        fit.addDrone(it)
    elif cat == "Charge":
        #Figure the currently selected module
        moduleStore = self.tvwModules.get_model()
        modPath = self.tvwModules.get_cursor()[0]
        if modPath == -1 or modPath == None:
            return

        iter = moduleStore.get_iter(modPath)
        mod, oldAmmo = moduleStore.get(iter, 9, 10)
        if mod == None:
            return

        if fitting.ammoFits(mod, it):
            fit.addAmmo(mod, it, oldAmmo)
    elif cat == "Implant":
        if it.group.name == "Booster": fit.addBooster(it)
        else: fit.addImplant(it)
    else:
        if fit.canAddModule(it):
            fit.addModule(it)
    
    #Manage recently used
    self.manageRecent(it)
    self.emit("fitting-changed")

def removeItem(self, source):
    #Get item being Removed
    itemStore = self.tvwModules.get_model()
    modPath = self.tvwModules.get_cursor()[0]
    if modPath == -1 or modPath == None:
        return

    iter = itemStore.get_iter(modPath)
    item = itemStore.get(iter, 9)[0]
    if item == None: return
    ammo = itemStore.get(iter, 10)[0]

    #Get current fit
    fit = self.getActiveFit()
    if fit == None: return
    fit.removeModule(item, ammo)
    #Manage recently used
    self.manageRecent(item)
    self.emit("fitting-changed")
    self.tvwModules.set_cursor(modPath[0])

def sortBySlot(self, store, iter1, iter2):
    slot1 = store.get_value(iter1, 11)
    slot2 = store.get_value(iter2, 11)
    if slot1 == None: return 1
    if slot2 == None: return -1
    slotPrio1 = fitting.slotTypes.index(slot1)
    slotPrio2 = fitting.slotTypes.index(slot2)
    if slotPrio1 > slotPrio2:
        return 1
    elif slotPrio1 < slotPrio2:
        return -1
    else:
        index1 = store.get_value(iter1, 13)
        index2 = store.get_value(iter2, 13)
        if index1 == -5 or index2 == -5: return index1 and 1 or -1
        return cmp(index1, index2)

def changeAmmo(self, source, ammo):
    fit = self.getActiveFit()
    if fit == None: return
    store = self.tvwModules.get_model()
    modules = []
    rows = self.tvwModules.get_selection().get_selected_rows()[1]
    for row in rows:
        iter = store.get_iter(row)
        modules.append(store.get_value(iter, 13))

    for index in modules:
        module, state, oldAmmo = fit.modules[index]
        fit.modules[index] = (module, state, copy.deepcopy(ammo))

    self.emit("fitting-changed")

def tvwModulesClicked(self, widget, event):
    stuff = widget.get_path_at_pos(int(event.x), int(event.y))
    fit   = self.getActiveFit()
    if not stuff or not fit:
        return True
    path, col      = stuff[0:2]
    path_in_select = False

    event_handlers = [
        [
            lambda: (event.button == 1 and (col != self.itemStateCol or not path_in_select)) or (event.button == 3 and not path_in_select),
            lambda i,w,e: widget.do_button_press_event(widget, event) and False
        ],
        [
            lambda: col != self.itemStateCol and event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS,
            lambda i,w,e: self.removeItem(widget) or True,
        ],
        [
            lambda: col == self.itemStateCol and event.type == gtk.gdk.BUTTON_PRESS and event.button in [1, 3],
            self.toggleModuleStates,
        ],
        [
            lambda: col != self.itemStateCol and event.button == 3,
            self.popupModuleContext,
        ]
    ]

    for handler in event_handlers:
        store, selected_rows = self.tvwModules.get_selection().get_selected_rows()
        path_in_select       = path in selected_rows
        info                 = [ fit, path, col, store, selected_rows, ]

        if handler[0]() and handler[1](info, widget, event):
            break
    return True

def toggleModuleStates(self, info, widget, event):
    fit, path, col, store, selected_rows = info
    fitting_changed = self._toggleModuleState(event, store, fit, path, None)

    proposed_state = None
    if fitting_changed:
        iter = store.get_iter(path)
        fit_idx = store.get_value(iter, 13)
        module, proposed_state, ammo = fit.modules[fit_idx]

    for row in selected_rows:
        if row == path: continue
        fitting_changed += self._toggleModuleState(event, store, fit, row, proposed_state)

    if fitting_changed: self.emit("fitting-changed")
    return fitting_changed == 0

def _toggleModuleState(self, event, store, fit, path, proposed_state):
    changed = 0
    iter    = store.get_iter(path)
    item    = store.get(iter, 9)[0]
    if not item or item.getModifiedAttribute("_slot") in ("rig", "subsystem"):
        return False

    fit_idx = store.get(iter, 13)[0]
    module, state, ammo = fit.modules[fit_idx]

    if proposed_state in (fitting.STATE_ACTIVE, fitting.STATE_INACTIVE, fitting.STATE_OFFLINE):
        if state != proposed_state:
            changed = 1
        state = proposed_state
    elif (proposed_state == fitting.STATE_OVERLOADED or event.button == 3) and "overload" in module.getType():
        if state != fitting.STATE_OVERLOADED:
            changed = 1
        state = fitting.STATE_OVERLOADED
    elif event.button == 1:
        changed = 1
        if state == fitting.STATE_OVERLOADED: state = fitting.STATE_ACTIVE
        elif state == fitting.STATE_ACTIVE: state = fitting.STATE_OFFLINE
        elif state == fitting.STATE_INACTIVE:
            if fit.canActivateModule(module): state = fitting.STATE_ACTIVE
            else: state = fitting.STATE_OFFLINE
        elif state == fitting.STATE_OFFLINE:
            if "active" in module.getType(): state = fitting.STATE_INACTIVE
            else: state = fitting.STATE_ACTIVE
        else: state += 1

    fit.modules[fit_idx] = module, state, ammo
    return changed

def popupModuleContext(self, info, widget, event):
    fit, path, col, store, selected_rows = info
    grps = set()
    ammoGrps = set()
    modules = []
    ammos = []
    store = widget.get_model()
    for row in selected_rows:
        iter = store.get_iter(row)
        module, ammo = store.get(iter, 9, 10)
        if module:
            grps.add(module.groupID)
            modules.append(module)
        if ammo:
            ammoGrps.add(ammo.groupID)
            ammos.append(ammo)

    children = self.fittedModuleMenu.get_children()
    if len(grps) == 1 and not self.fittedModuleGroupEntry in children:
        self.fittedModuleMenu.add(self.fittedModuleGroupEntry)
    elif len(grps) != 1 and self.fittedModuleGroupEntry in children:
        self.fittedModuleMenu.remove(self.fittedModuleGroupEntry)

    if len(modules) >= 1 and not self.fittedModuleStatsEntry in children:
        self.fittedModuleMenu.add(self.fittedModuleStatsEntry)
    elif len(modules) == 0 and self.fittedModuleStatsEntry in children:
        self.fittedModuleMenu.remove(self.fittedModuleStatsEntry)

    if len(ammoGrps) == 1 and not self.fittedAmmoGroupEntry in children:
        self.fittedModuleMenu.add(self.fittedAmmoGroupEntry)
    elif len(ammoGrps) == 0 and self.fittedAmmoGroupEntry in children:
        self.fittedModuleMenu.remove(self.fittedAmmoGroupEntry)

    if len(ammos) >= 1 and not self.fittedAmmoStatsEntry in children:
        self.fittedModuleMenu.add(self.fittedAmmoStatsEntry)
    elif len(ammos) == 0 and self.fittedAmmoStatsEntry in children:
        self.fittedModuleMenu.remove(self.fittedAmmoStatsEntry)

    m = self.generateAmmoMenu(modules, self.changeAmmo)
    if m:
        if not self.fittedAmmoChangeEntry in children:
            self.fittedModuleMenu.add(self.fittedAmmoChangeEntry)
        self.fittedAmmoChangeEntry.set_submenu(m)
    elif self.fittedAmmoChangeEntry in children:
        self.fittedModuleMenu.remove(self.fittedAmmoChangeEntry)

    self.fittedModuleMenu.popup(None, None, None, event.button, event.time)
    self.fittedModuleMenu.show_all()

def tvwModulesKeyPress(self, widget, event):
    fit = self.getActiveFit()

    event_handlers = [
        [ lambda: fit and event.keyval == 65535, self.deleteModules ],
    ]

    for handler in event_handlers:
        if handler[0]() and handler[1](fit, widget, event):
            break
    return True

def deleteModules(self, fit, widget, event):
    store = self.tvwModules.get_model()
    rows = self.tvwModules.get_selection().get_selected_rows()[1]
    modules = []
    for row in rows:
        iter = store.get_iter(row)
        modules.append(store.get_value(iter, 13))
    modules.sort(reverse=True)
    for i in modules: del fit.modules[i]
    self.emit("fitting-changed")
    return True
