#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
import gtk
from model import item, fitting
from gui import pixbufLoader
from gui.itemStatsWindow import itemStatsWindow

def init(self):
    self.connect("fitting-changed", self.handleFittingChangedGang)
    self.tvwGang.connect("button-press-event", self.tvwGangClicked)
    self.tvwItems.connect("button-press-event", self.manageGangItemEntry)
        
    #active [for checkbox], Icon, name, level [if any]
    #description [for tooltip], item
    store = gtk.ListStore(bool, gtk.gdk.Pixbuf, str, str, str, object)
    store.set_sort_func(5, self.gangSortMethod)
    store.set_sort_column_id(5, gtk.SORT_ASCENDING)
    self.tvwGang.set_model(store)
    self.tvwGang.set_tooltip_column(4)
    
    activeRenderer = gtk.CellRendererToggle()
    iconRenderer = gtk.CellRendererPixbuf()
    nameRenderer = gtk.CellRendererText()
    levelRenderer = gtk.CellRendererText()
    
    self.gangStateCol = gtk.TreeViewColumn()
    colItem = gtk.TreeViewColumn("Item")
    colLevel = gtk.TreeViewColumn("Level")
    
    self.gangStateCol.pack_start(activeRenderer, False)
    colItem.pack_start(iconRenderer, False)
    colItem.pack_start(nameRenderer, True)
    colLevel.pack_start(levelRenderer, False)
    
    self.gangStateCol.add_attribute(activeRenderer, 'active', 0)
    colItem.add_attribute(iconRenderer, 'pixbuf', 1)
    colItem.add_attribute(nameRenderer, 'text', 2)
    colLevel.add_attribute(levelRenderer, 'text', 3)
    activeRenderer.set_property("activatable", True)
    
    self.tvwGang.append_column(self.gangStateCol)
    self.tvwGang.append_column(colItem)
    self.tvwGang.append_column(colLevel)
    
    #Generate menus we need
    skillBookPixbuf = pixbufLoader.getPixbuf("skill")
    box = self.generateHBox(gtk.image_new_from_pixbuf(skillBookPixbuf),
                            gtk.Label("Autoset All"))
    self.gangMenu = self.generateMenu({gtk.Label("Stats") : self.gangItemStats,
                                       box : self.autoSetGangAll})
    self.gangGroupEntry = self.generateMenuItem(gtk.Label("Market Group"), self.gangItemGroup)
    self.gangLevelEntries = []
    for i in range(6):
        box = gtk.HBox()
        skillBookImage = gtk.Image()
        skillBookImage.set_from_pixbuf(skillBookPixbuf)
        lbl = gtk.Label("Level %d" % i)
        lbl.set_alignment(0, 0.5)
        box.pack_start(skillBookImage, False, False)
        box.pack_start(lbl, True, True)
        self.gangLevelEntries.append(self.generateMenuItem(box, self.gangItemSetLevel, i))
    
    self.gangAddEntry = self.generateMenuItem(gtk.Label("Add as gang bonus"), self.addGangModule)
    box = self.generateHBox(gtk.image_new_from_pixbuf(skillBookPixbuf),
                            gtk.Label("Autoset"))
    self.gangAutosetEntry = self.generateMenuItem(box, self.autoSetGang)
    shipPixbuf = pixbufLoader.getPixbuf("ships")
    box = self.generateHBox(gtk.image_new_from_pixbuf(shipPixbuf),
                            gtk.Label("Add fit"))
    self.gangAddFitEntry = self.generateMenuItem(box, None)
    self.gangMenu.append(self.gangAddFitEntry)

def handleFittingChangedGang(self, *stuff):
    fit = self.getActiveFit()
    if fit == None: self.tvwGang.get_model().clear()
    else: self.updateGangBrowser()
    
def manageGangItemEntry(self, widget, event):
    widget.do_button_press_event(widget, event)
    children = self.itemBrowserMenu.get_children()
    item = self.getSelectedItem()
    fit = self.getActiveFit()
    if item == None or fit == None: return
    if "gang" in item.getType() and not self.gangAddEntry in children and fit != None:
        self.itemBrowserMenu.add(self.gangAddEntry)
    elif (not "gang" in item.getType() or fit == None) and self.gangAddEntry in children:
        self.itemBrowserMenu.remove(self.gangAddEntry)

def getSelectedGangItem(self):
    store = self.tvwGang.get_model()
    select = self.tvwGang.get_cursor()[0]
    iter = store.get_iter(select)
    return store.get_value(iter, 5)
    
def gangItemStats(self, *stuff):
    itemStatsWindow(self.getSelectedGangItem(), self.getActiveFit(), None)
def gangItemGroup(self, *stuff): self.expandToItem(self.getSelectedGangItem())
def gangItemSetLevel(self, source, level):
    item = self.getSelectedGangItem()
    self.getActiveFit().gangModules[item]["level"] = level
    self.emit("fitting-changed")

def updateGangBrowser(self):
    fit = self.getActiveFit()
    if fit == None: return
    store = self.tvwGang.get_model()
    select = self.tvwGang.get_cursor()[0]
    #If store is not set we got called early, let's gtfo
    if store == None: return
    store.clear()
    for module, overrides in fit.gangModules.iteritems():
        if overrides and "level" in overrides:
            level = "%d" % overrides["level"]
        else:
            level = None
            
        active = module in fit.activeGangModules
        if isinstance(module, item.item):
            if module.group.category.name == "Ship": pixbuf = pixbufLoader.getPixbuf("ships")
            else: pixbuf = pixbufLoader.getPixbuf(module.icon, True, "16x16")
            name = module.name
            description = module.description
        else:
            pixbuf = pixbufLoader.getPixbuf("ships")
            fits = self.fits[module.ship.group.name][module.ship]
            description = "%s - %s" % (module.ship.name, module.ship.group.name)
            for n, fit in fits.iteritems():
                if fit == module:
                    name = n
                    break
        store.append((active, pixbuf, name, level, description, module))
    #We're done, set the cursor back to where it was
    if select != None and select != -1: self.tvwGang.set_cursor(select)
    
def gangSortMethod(self, treemodel, iter1, iter2):
    item1 = treemodel.get(iter1, 5)[0]
    item2 = treemodel.get(iter2, 5)[0]
    if item1 and not item2:
        return -1
    elif item2 and not item1:
        return 1
    elif not item2 and not item1:
        return 0
    else:
        if isinstance(item1, item.item):
            i1Cat = item1.group.category.name
            i1Type = item1.getType()
        else: i1Cat = "ship"
        if isinstance(item2, item.item):
            i2Cat = item2.group.category.name
            i2Type = item2.getType()
        else: i2Cat = "ship"
        
        
        if (i1Cat == "ship" and i2Cat != "ship") or \
           (not "Skill" in i1Cat and "Skill" in i2Cat) or \
           ("ondemand" in i1Type and not "ondemand" in i2Type):
            return -1
        elif (i1Cat != "ship" and i2Cat == "ship") or \
             ("Skill" in i1Cat and not "Skill" in i2Cat) or \
             (not "ondemand" in i1Type and "ondemand" in i2Type):
            return 1
        else:
            return cmp(item1.name, item2.name)

def tvwGangClicked(self, widget, event):
    widget.do_button_press_event(widget, event)
    path = widget.get_cursor()[0]
    if path == -1 or path == None: return
    store = widget.get_model()
    iter = store.get_iter(path)
    state, mod, level = store.get(iter, 0, 5, 3)
    if event.button == 1:
        col = widget.get_path_at_pos(int(event.x), int(event.y))
        if col: col = col[1]
        else: return
        fit = self.getActiveFit()
        if fit == None: return
        if event.type == gtk.gdk.BUTTON_PRESS and col == self.gangStateCol:
            state = not state
            store.set(iter, 0, state)
            if not state: del fit.activeGangModules[mod]
            else: fit.activeGangModules[mod] = {}
            self.emit("fitting-changed")
            
        elif event.type == gtk.gdk._2BUTTON_PRESS and col != self.gangStateCol:
            if (isinstance(mod, fitting.fitting) or \
                mod.group.category.name != "Skill" or "ondemand" in mod.getType()) and \
                mod in fit.extraGangModules:
                del fit.extraGangModules[mod]
                if mod in fit.activeGangModules: del fit.activeGangModules[mod]
                self.emit("fitting-changed")
    elif event.button == 3:
        menu = self.generateFitMenu(self.addGangFit)
        self.gangAddFitEntry.set_submenu(menu)
        fitMenu = self.generateFitMenu(self.addGangFit)
        children = self.gangMenu.get_children()
        if level and not self.gangLevelEntries[0] in children:
            for menuItem in self.gangLevelEntries: self.gangMenu.add(menuItem)
        elif not level and self.gangLevelEntries[0] in children:
            for menuItem in self.gangLevelEntries: self.gangMenu.remove(menuItem)
        if mod.group.category.name != "Skill" and not self.gangGroupEntry in children:
            self.gangMenu.add(self.gangGroupEntry)
        elif mod.group.category.name == "Skill" and self.gangGroupEntry in children:
            self.gangMenu.remove(self.gangGroupEntry)
        
        self.gangMenu.popup(None, None, None, event.button, event.time)
        self.gangMenu.show_all()


def addGangModule(self, *stuff):
    item = self.getSelectedItem()
    fit = self.getActiveFit()
    if fit == None: return
    fit.extraGangModules[item] = {"state" : fitting.STATE_ACTIVE}
    self.emit("fitting-changed")

def addGangFit(self, widget, fit):
    activeFit = self.getActiveFit()
    if activeFit == None: return
    activeFit.extraGangModules[fit] = {}
    self.emit("fitting-changed")
    
def gangChangeLevel(self, widget, newLevel):
    fit = self.getActiveFit()
    if fit == None: return
    path = self.tvwGang.get_cursor()[0]
    store = self.tvwGang.get_model()
    iter = store.get_iter(path)
    state, mod, level = store.get(iter, 0, 5, 3)
    fit.gangModules[mod]["level"] = newLevel
    self.emit("fitting-changed")

def autoSetGang(self, *stuff):
    fit = self.getActiveFit()
    char = self.getActiveCharacter()
    path = self.tvwGang.get_cursor()[0]
    store = self.tvwGang.get_model()
    iter = store.get_iter(path)
    mod = store.get_value(iter, 5)
    fit.gangModules[mod]["level"] = mod in char.skills and char.skills[mod] or 0
    self.emit("fitting-changed")

def autoSetGangAll(self, *stuff):
    fit = self.getActiveFit()
    char = self.getActiveCharacter()
    for mod, overrides in fit.gangModules.iteritems():
        if mod.group.category.name == "Skill":
            overrides["level"] = mod in char.skills and char.skills[mod] or 0
    self.emit("fitting-changed")
