#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
import gtk
from model import marketGroup, graphic, category, group, item
from gui import pixbufLoader
import copy

def init(self):
    #Load favorites
    if not "favoriteItems" in self.guiSettings: self.guiSettings["favoriteItems"] = set()
    self.favoriteItems = self.guiSettings["favoriteItems"]
    #Check if favoriteItems is still in the old format and if so, convert
    newFavs = set()
    for i in self.favoriteItems:
        if isinstance(i, item.item):
            newFavs.add(i.ID)
    if len(newFavs) > 0: self.favoriteItems = newFavs
            
    if not "recentItems" in self.guiSettings: self.guiSettings["recentItems"] = []
    self.recentItems = self.guiSettings["recentItems"]
    for i in range(len(self.recentItems)):
        if isinstance(self.recentItems[i], item.item):
            self.recentItems[i] = self.recentItems[i].ID
    
    store = gtk.TreeStore(gtk.gdk.Pixbuf, str, object)
    store.set_sort_column_id(1, gtk.SORT_ASCENDING)
    store.set_sort_func(1, self.groupSort)
    self.tvwGroups.set_model(store)
    self.tvwGroups.set_tooltip_column(2)
    self.tvwGroups.set_headers_visible(False)
    
    imageRenderer = gtk.CellRendererPixbuf()
    nameRenderer = gtk.CellRendererText()
    col = gtk.TreeViewColumn()

    col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
    col.pack_start(imageRenderer, False)
    col.pack_start(nameRenderer, True)
    col.add_attribute(imageRenderer, 'pixbuf', 0)
    col.add_attribute(nameRenderer, 'text', 1)
    self.tvwGroups.append_column(col)
    #Modules
    mGroupMods = marketGroup.getMarketGroup(name = "Ship Equipment")
    self.buildGroups(mGroupMods, None, True)
    #Rigs
    mGroupRigs = marketGroup.getMarketGroup(name = "Rigs")
    self.buildGroups(mGroupRigs, None, True)
    #Drones
    mGroupDrones = marketGroup.getMarketGroup(name = "Drones")
    self.buildGroups(mGroupDrones, None, True)
    #Ammo
    mGroupAmmo = marketGroup.getMarketGroup(name = "Ammunition & Charges")
    self.buildGroups(mGroupAmmo, None, True)
    #Subsystems. Can't look by market group name here or there's a chance we'll get the skills
    mGroupSubsystems = marketGroup.getMarketGroup(ID = 1112)
    self.buildGroups(mGroupSubsystems, None, True)
    #Implants & Boosters
    mGroupImplantsBoosters = marketGroup.getMarketGroup(name = "Implants & Boosters")
    iter = self.addGroup(mGroupImplantsBoosters.name, mGroupImplantsBoosters, None)
    cat = category.getCategory(name = "Implant")
    grps = group.getGroupsByCategory(cat.ID)
    for grp in grps:
        if grp.published == 1:
            if grp.name == "Booster":
                pixbuf = pixbufLoader.getPixbuf("booster")
            else:
                pixbuf = pixbufLoader.getPixbuf("hardwiring")
            store.append(iter, (pixbuf, grp.name, grp))
    
    #Favorites
    store.append(None, (None, "Favorites", "favorite"))
    #Recently used
    store.append(None, (None, "Recently Used", "recent"))
    
    #Connect events
    self.tvwGroups.connect("cursor-changed", self.groupSelected)
    self.tvwGroups.connect("button-press-event", self.toggleGroup)

def getSelectedGroup(self):
    store = self.tvwGroups.get_model()
    path = self.tvwGroups.get_cursor()[0]
    if path == None or path == -1: return None
    iter = store.get_iter(path)
    return store.get_value(iter, 2)

def groupSort(self, store, iter1, iter2):
    #Put stuff of type group or marketGroup at top, after then the rest
    grp1, grp2 = store.get_value(iter1, 2), store.get_value(iter2, 2)
    prio1 = isinstance(grp1, marketGroup.marketGroup) or isinstance(grp1, group.group)
    prio2 = isinstance(grp2, marketGroup.marketGroup) or isinstance(grp2, group.group)
    if prio1 != prio2: return prio1 and -1 or 1
    else:
        name1, name2 = store.get_value(iter1, 1), store.get_value(iter2, 1)
        return cmp(name1, name2)
        

def toggleGroup(self, widget, event):
    widget.do_button_press_event(widget, event)
    if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
        path = self.tvwGroups.get_cursor()[0]
        if path != None and path != -1:
            if self.tvwGroups.row_expanded(path):
                self.tvwGroups.collapse_row(path)
            else: self.tvwGroups.expand_row(path, False)
        
def expandToItem(self, module):
    if not module.marketGroupID: baseModule = item.getBase(module.ID) or module
    else: baseModule = module
    path = self.lookForGroup(baseModule.marketGroup) or self.lookForGroup(baseModule.group)
    self.tvwGroups.expand_to_path(path)
    self.tvwGroups.set_cursor(path)
    metaGroup = module.metaGroupID
    if metaGroup == 4: tbn = self.tbnFaction
    elif metaGroup == 5: tbn = self.tbnOfficer
    elif metaGroup == 6: tbn = self.tbnComplex
    else: tbn = self.tbnNormal
    for candidate in (self.tbnNormal, self.tbnFaction, self.tbnComplex, self.tbnOfficer):
        if candidate == tbn: candidate.set_active(True)
        else: candidate.set_active(False)
    
        
def lookForGroup(self, neededGroup, iter = None, store = None, i = 0):
    store = store or self.tvwGroups.get_model()
    iter = iter or store.get_iter((0,))
    while iter != None:
        group = store.get_value(iter, 2)
        if group == neededGroup: return store.get_path(iter)
        iterChild = store.iter_children(iter)
        if iterChild != None:
            inGroup = self.lookForGroup(neededGroup, iterChild, store, i + 1)
            if inGroup != None:
                return inGroup
        
        iter = store.iter_next(iter)

def buildGroups(self, parentGroup, parentIter, includeParent = False):
    store  = self.tvwGroups.get_model()
    if includeParent:
        parentIter = self.addGroup(parentGroup.name, parentGroup, parentIter)
    
    if parentGroup.children:
        for name, mGroup in parentGroup.children.iteritems():
            if name != "Deployable Equipment":
                iter = self.addGroup(name, mGroup, parentIter)
                self.buildGroups(mGroup, iter, False)
        
    if includeParent: return parentIter

def groupSelected(self, *stuff):
    if self.itemSearchActive:
        self.itemSearchActive = False
        self.tbnNormal.set_active(True)
        for tbn in (self.tbnFaction, self.tbnComplex, self.tbnOfficer):
            tbn.set_active(False)
            
    store = self.tvwGroups.get_model()
    path = self.tvwGroups.get_cursor()[0]
    if path == None or path == -1: return
    iter = store.get_iter(path)
    grp = store.get_value(iter, 2)
    
    #Normal items, those are in market groups so it's easy
    store = self.tvwItems.get_model()
    store.clear()
    if isinstance(grp, marketGroup.marketGroup):
        items = item.getItemsByGroup(grp)     
        if self.itemBrowserSlotCol in self.tvwItems.get_columns():
            self.tvwItems.remove_column(self.itemBrowserSlotCol)
    elif isinstance(grp, group.group):
        items = item.getItemsByGroup(group = grp)
        if not self.itemBrowserSlotCol in self.tvwItems.get_columns():
            self.tvwItems.insert_column(self.itemBrowserSlotCol, 0)
    elif grp == "recent":
        items = copy.copy(self.recentItems)
        for i in range(len(items)): items[i] = item.getItem(ID = items[i])
    elif grp == "favorite":
        items = copy.copy(self.favoriteItems)
        for i in range(len(items)): items[i] = item.getItem(ID = items[i])
    
    metaIds = set()
    for i in items:
        if i.published == 1:            
            #append the item itself
            pixbuf = pixbufLoader.getPixbuf(i.icon, True, "16x16")
            if i.group.category.name == "Implant":
                if i.group.name == "Booster":
                    slot = "%d" % i.getModifiedAttribute("boosterness")
                else:
                    slot = "%d" % i.getModifiedAttribute("implantness")
            else:
                slot = None
            if self.itemFilter(i):
                store.append((pixbuf, i.name, i.description, i, slot))
            
            #handle variations
            for v in item.getVariations(i):
                if v not in items:
                    metaIds.add(v.metaGroupID)
                    if self.itemFilter(v): store.append((pixbuf, v.name, v.description, v, slot))
    
    if 4 in metaIds: self.tbnFaction.set_sensitive(True)
    else: self.tbnFaction.set_sensitive(False)
    if 5 in metaIds: self.tbnOfficer.set_sensitive(True)
    else: self.tbnOfficer.set_sensitive(False)
    if 6 in metaIds: self.tbnComplex.set_sensitive(True)
    else: self.tbnComplex.set_sensitive(False)
    
def addGroup(self, name, mGroup, iter):
    store = self.tvwGroups.get_model()
    if mGroup.graphicID != None:
        icon = graphic.getIconName(mGroup.graphicID)
        pixbuf = pixbufLoader.getPixbuf(icon, True, "16x16")
    else:
        pixbuf = None
            
    return store.append(iter, (pixbuf, name, mGroup))