#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
import gtk
import copy
import gobject
from model import item, fitting
from gui import pixbufLoader
from gui.itemStatsWindow import itemStatsWindow
import groupBrowser

depencies = (groupBrowser,)
def init(self):
    self.window.connect("key-press-event", self.checkControlPress)
    self.window.connect("key-release-event", self.checkControlRelease)
    self.itemSearchActive = False
    self.filterMultiMode = False
    for tbn in (self.tbnNormal, self.tbnFaction, self.tbnComplex, self.tbnOfficer):
        tbn.connect("button-press-event", self.handleFilterPress)
        if tbn != self.tbnNormal: tbn.set_sensitive(False)
        tbn.connect("toggled", self.refilterItems)
        
    self.entrySearchItem.connect("activate", self.scheduleSearch)
    store = gtk.ListStore(gtk.gdk.Pixbuf, str, str, object, str)
    self.tvwItems.set_model(store)
    self.tbnNormal.set_active(True)
    self.tvwItems.set_tooltip_column(2)
    self.tvwItems.set_headers_visible(False)
    store.set_sort_column_id(1, gtk.SORT_ASCENDING)
    store.set_sort_func(1, self.sortItemBrowser)
    iconRenderer = gtk.CellRendererPixbuf()
    nameRenderer = gtk.CellRendererText()
    col = gtk.TreeViewColumn()
    col.set_sizing(gtk.TREE_VIEW_COLUMN_AUTOSIZE)
    col.pack_start(iconRenderer, False)
    col.pack_start(nameRenderer, True)
    col.add_attribute(iconRenderer, 'pixbuf', 0)
    col.add_attribute(nameRenderer, 'text', 1)
    self.tvwItems.append_column(col)
    self.tvwItems.connect("button-press-event", self.itemsClicked)
    
    self.itemBrowserSlotCol = gtk.TreeViewColumn("Slot")
    slotRenderer = gtk.CellRendererText()
    self.itemBrowserSlotCol.pack_start(slotRenderer, False)
    self.itemBrowserSlotCol.add_attribute(slotRenderer, 'text', 4)
    self.itemBrowserSlotColUsed = False
    
    self.entrySearchItem.connect("changed", self.scheduleSearch)
    self.btnSearchItem.connect("clicked", self.scheduleSearch)

    self.itemBrowserMenu = self.generateMenu({gtk.Label("Stats") : self.itemStats})
    self.itemGroupMenuEntry = self.generateMenuItem(gtk.Label("Market Group"), self.itemGroup)
    self.itemBrowserLoadAmmoEntry = self.generateMenuItem(gtk.Label("Load Charge"), self.itemLoadAmmo)
    self.itemAddFavorites = self.generateMenuItem(gtk.Label("Add to favorites"), self.addFavorite)
    self.itemRemoveFavorites = self.generateMenuItem(gtk.Label("Remove from favorites"), self.removeFavorite)
    
    self.pendingSearch = None

def refilterItems(self, *stuff):
    if not self.itemSearchActive: self.groupSelected()
    else: self.scheduleSearch()

def itemFilter(self, item):
    if item == None: return False
    if item.marketGroupID == 0: return self.tbnNormal.get_active()
    elif item.metaGroupID == 4: return self.tbnFaction.get_active()
    elif item.metaGroupID == 5: return self.tbnOfficer.get_active()
    elif item.metaGroupID == 6: return self.tbnComplex.get_active()
    else: return self.tbnNormal.get_active()

def checkControlPress(self, widget, event):
    if gtk.gdk.keyval_name(event.keyval)[0:7] == "Control": self.filterMultiMode = True

def checkControlRelease(self, widget, event):
    if gtk.gdk.keyval_name(event.keyval)[0:7] == "Control": self.filterMultiMode = False
    
def handleFilterPress(self, widget, event):
    tbns = (self.tbnNormal, self.tbnFaction, self.tbnComplex, self.tbnOfficer)
    if not self.filterMultiMode:
        for tbn in tbns: tbn.set_active(False)    

def addFavorite(self, *stuff): self.favoriteItems.add(self.getSelectedItem().ID)
def removeFavorite(self, *stuff):
    self.favoriteItems.remove(self.getSelectedItem().ID)
    self.groupSelected(self.tvwGroups)

def sortItemBrowser(self, store, iter1, iter2):
    grp = self.getSelectedGroup()
    slot1, name1 = store.get(iter1, 4, 1)
    slot2, name2 = store.get(iter2, 4, 1)
    if grp == "recent" or grp == "favorite" or self.itemSearchActive:
        item1 = store.get_value(iter1, 3)
        item2 = store.get_value(iter2, 3)
        if not item1 or not item2: return item1 and -1 or 1
        if grp == "recent" and not self.itemSearchActive:
            if item1.ID in self.recentItems and item2.ID in self.recentItems:
                return cmp(self.recentItems.index(item2.ID), self.recentItems.index(item1.ID))
            else: return 0
        else: return cmp(item1.group.name, item2.group.name) or cmp(name1, name2)
    else:
        if slot1 != None: slot1 = int(slot1)
        if slot2 != None: slot2 = int(slot2)
        if slot1 != None or slot2 != None:
            sort = cmp(slot1, slot2)
            if sort == 0: sort = cmp(name1, name2)
        else: sort = cmp(name1, name2)
        return sort
    
def getSelectedItem(self):
    store = self.tvwItems.get_model()
    select = self.tvwItems.get_cursor()[0]
    if select == None or select == -1: return
    iter = store.get_iter(select)
    return store.get_value(iter, 3)

def itemStats(self, *stuff): itemStatsWindow(self.getSelectedItem(), None, None)

def itemGroup(self, *stuff): self.expandToItem(self.getSelectedItem())

def itemLoadAmmo(self, *stuff):
    ammo = self.getSelectedItem()
    fit = self.getActiveFit()
    for i in range(len(fit.modules)):
        target, state, currAmmo = fit.modules[i]
        if fitting.ammoFits(target, ammo):
            fit.modules[i] = (target, state, copy.deepcopy(ammo))
    self.emit("fitting-changed")
    
def buildItems(self, parentGroup, parentIter):
    store  = self.tvwItems.get_model()
    items = item.getItemsByGroup(marketGroup = parentGroup)
    for i in items:
        if i.icon != None: pixbuf = pixbufLoader.getPixbuf(i.icon, True, "16x16")
        else: pixbuf = None
        store.append(parentIter, (pixbuf, i.name, i.description, i))
        
def itemsClicked(self, widget, event):
    widget.do_button_press_event(widget, event)
    if event.button == 1:
        if event.type == gtk.gdk._2BUTTON_PRESS:
            self.addItem(widget)
    if event.button == 3:
        #We need to figure if a gang module was right clicked
        store = widget.get_model()
        path = widget.get_cursor()[0]
        if path == None or path == -1: return
        iter = store.get_iter(path)
        module = store.get(iter, 3)[0]
        module = item.getItem(ID = module.ID)
        modType = module.getType()
        #Check if the selected item fits as ammo in any fitted module
        ammoShow = False
        fit = self.getActiveFit()
        if fit != None:
            for i in range(len(fit.modules)):
                currModule, state, currAmmo = fit.modules[i]
                if fitting.ammoFits(currModule, module):
                    ammoShow = True
                    break

        children = self.itemBrowserMenu.get_children()
        grp = self.getSelectedGroup()
        favoritesActive = grp == "favorite"
        if not favoritesActive and not self.itemAddFavorites in children:
            self.itemBrowserMenu.add(self.itemAddFavorites)
        elif favoritesActive and self.itemAddFavorites in children:
            self.itemBrowserMenu.remove(self.itemAddFavorites)
            
        if favoritesActive and not self.itemRemoveFavorites in children:
            self.itemBrowserMenu.add(self.itemRemoveFavorites)
        elif not favoritesActive and self.itemRemoveFavorites in children:
            self.itemBrowserMenu.remove(self.itemRemoveFavorites)
        
        if ammoShow and not self.itemBrowserLoadAmmoEntry in children:
            self.itemBrowserMenu.add(self.itemBrowserLoadAmmoEntry)
        elif not ammoShow and self.itemBrowserLoadAmmoEntry in children:
            self.itemBrowserMenu.remove(self.itemBrowserLoadAmmoEntry)
            
        if (self.itemSearchActive or grp == "favorite" or grp == "recent") \
           and not self.itemGroupMenuEntry in children:
            self.itemBrowserMenu.add(self.itemGroupMenuEntry)
        elif not self.itemSearchActive and grp != "favorite" and grp != "recent"\
           and self.itemGroupMenuEntry in children:
            self.itemBrowserMenu.remove(self.itemGroupMenuEntry)
        
        self.itemBrowserMenu.popup(None, None, None, event.button, event.time)
        self.itemBrowserMenu.show_all()

def scheduleSearch(self, *stuff):
    text = self.entrySearchItem.get_text().encode("ascii", "replace")
    if len(text) >= 3:
        if self.pendingSearch: gobject.source_remove(self.pendingSearch)
        self.pendingSearch = gobject.idle_add(self.searchItem, text)
    else: 
        self.tvwItems.get_model().clear()
        for tbn in (self.tbnFaction, self.tbnComplex, self.tbnOfficer): tbn.set_sensitive(False)
        self.tbnNormal.set_active(True)
    
def searchItem(self, text):
    self.pendingSearch = None
    if not self.itemSearchActive:
        self.itemSearchActive = True
        self.tvwGroups.get_selection().unselect_all()
        for tbn in (self.tbnNormal, self.tbnFaction, self.tbnComplex, self.tbnOfficer):
            tbn.set_active(True)
    
    itemStore = self.tvwItems.get_model()
    itemStore.clear()
    store = self.tvwGroups.get_model()
    items = item.searchItem(text)
    goodCats = ("Drone",
                "Module",
                "Subsystem",
                "Charge",
                "Implant",
                "Booster")
    
    metaIds = set()
    for i in items:
        if i.published == 1:
            metaIds.add(i.metaGroupID)
            if i.group.category.name in goodCats and self.itemFilter(i):
                pixbuf = pixbufLoader.getPixbuf(i.icon, True, "16x16")
                itemStore.append((pixbuf, i.name, i.description, i, None))
                
    if 4 in metaIds: self.tbnFaction.set_sensitive(True)
    else: self.tbnFaction.set_sensitive(False)
    if 5 in metaIds: self.tbnOfficer.set_sensitive(True)
    else: self.tbnOfficer.set_sensitive(False)
    if 6 in metaIds: self.tbnComplex.set_sensitive(True)
    else: self.tbnComplex.set_sensitive(False)
    return False

