#Copyright 2009 Diego Duclos
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.
import gtk
from model import item, fitting
from gui import pixbufLoader
from gui.itemStatsWindow import itemStatsWindow

def init(self):
    self.connect("fitting-changed", self.handleFittingChangedProj)
    self.tvwProj.connect("button-press-event", self.tvwProjClicked)
    self.tvwItems.connect("button-press-event", self.manageProjectedItemEntry)
        
    #State, pixbuf, amount, name, description, module
    store = gtk.ListStore(bool, gtk.gdk.Pixbuf, str, str, str, object)
    self.tvwProj.set_model(store)
    self.tvwProj.set_tooltip_column(4)
    activeRenderer = gtk.CellRendererToggle()
    activeRenderer.set_property("activatable", True)
    pixbufRenderer = gtk.CellRendererPixbuf()
    amountRenderer = gtk.CellRendererText()
    nameRenderer = gtk.CellRendererText()
    
    nameCol = gtk.TreeViewColumn('Item')
    self.projStateCol = gtk.TreeViewColumn()
    
    self.projStateCol.pack_start(activeRenderer, False)
    nameCol.pack_start(pixbufRenderer, False)
    nameCol.pack_start(amountRenderer, False)
    nameCol.pack_start(nameRenderer, False)
    
    self.projStateCol.add_attribute(activeRenderer, 'active', 0)
    nameCol.add_attribute(pixbufRenderer, 'pixbuf', 1)
    nameCol.add_attribute(amountRenderer, 'text', 2)
    nameCol.add_attribute(nameRenderer, 'text', 3)
    
    self.tvwProj.append_column(self.projStateCol)
    self.tvwProj.append_column(nameCol)
    
    #Generate menus
    self.projectedMenu = self.generateMenu({})
    self.projectedViewStatsEntry = self.generateMenuItem(gtk.Label("Stats"), self.projItemStats)
    self.projectedViewGroupEntry = self.generateMenuItem(gtk.Label("Market Group"), self.projItemGroup)
    self.projectedAddEntry = self.generateMenuItem(gtk.Label("Add as projected effect"), self.addProjModule)
    shipPixbuf = pixbufLoader.getPixbuf("ships")
    box = self.generateHBox(gtk.image_new_from_pixbuf(shipPixbuf), gtk.Label("Add Fit"))
    self.projectedAddFitEntry = self.generateMenuItem(box, None)
    self.projectedMenu.append(self.projectedAddFitEntry)

def handleFittingChangedProj(self, *stuff):
    fit = self.getActiveFit()
    if fit == None: self.tvwProj.get_model().clear()
    else: self.updateProjectedEffects()
    
def projItemStats(self, *stuff):
    effect = self.getSelectedProjEffect()
    if isinstance(effect, fitting.fitting):
        effect = effect.ship
        icon = pixbufLoader.getPixbuf(effect.race)
    else: icon = None
    itemStatsWindow(effect, self.getActiveFit(), icon)
    
def projItemGroup(self, *stuff): self.expandToItem(self.getSelectedProjEffect())

def getSelectedProjEffect(self):
    store = self.tvwProj.get_model()
    select = self.tvwProj.get_cursor()[0]
    if select == -1 or select == None: return
    iter = store.get_iter(select)
    return store.get_value(iter, 5)

def manageProjectedItemEntry(self, widget, event):
    widget.do_button_press_event(widget, event)
    children = self.itemBrowserMenu.get_children()
    item = self.getSelectedItem()
    if item == None: return
    fit = self.getActiveFit()
    if fit == None: return
    if "projected" in item.getType() and not self.projectedAddEntry in children and fit != None:
        self.itemBrowserMenu.add(self.projectedAddEntry)
        self.projectedAddEntry.show_all()
    elif (not "projected" in item.getType() or fit == None) and self.projectedAddEntry in children:
        self.itemBrowserMenu.remove(self.projectedAddEntry)
 
def updateProjectedEffects(self):
    fit = self.getActiveFit()
    if fit == None: return
    store = self.tvwProj.get_model()
    select = self.tvwProj.get_cursor()[0]
    store.clear()
    for module, (amount, active, overrides) in fit.projEffects.iteritems():
        if isinstance(module, item.item):
            icon = pixbufLoader.getPixbuf(module.icon, True, "16x16")
            name = module.name
            description = module.description
        else:
            icon = pixbufLoader.getPixbuf("ships")
            name = None
            if module.ship.group.name in self.fits and module.ship in self.fits[module.ship.group.name]:
                fits = self.fits[module.ship.group.name][module.ship]
                description = "%s - %s" % (module.ship.name, module.ship.group.name)
                for n, fit in fits.iteritems():
                    if fit == module:
                        name = n
                        break
            else: continue
        if name != None: store.append((active, icon, "%dx" % amount, name, description, module))
    
    l = len(fit.projEffects) -1
    if select != None and select != -1 and select[0] <= l and l != 0: self.tvwProj.set_cursor(select)
    
def tvwProjClicked(self, widget, event):
    widget.do_button_press_event(widget, event)
    col = widget.get_path_at_pos(int(event.x), int(event.y))
    if col and col[1] == self.projStateCol:
        col = col[1]
        if event.button == 1:
            fit = self.getActiveFit()
            if fit == None: return
            path = widget.get_cursor()[0]
            store = widget.get_model()
            iter = store.get_iter(path)
            module = store.get(iter, 5)[0]
            curr = fit.projEffects[module]
            if col == self.projStateCol and event.type == gtk.gdk.BUTTON_PRESS:
                fit.projEffects[module] = (curr[0],not curr[1], curr[2])
            
            self.emit("fitting-changed")
    elif event.button == 1 and event.type == gtk.gdk._2BUTTON_PRESS:
        effect = self.getSelectedProjEffect()
        fit = self.getActiveFit()
        if fit == None or effect == None: return
        curr = fit.projEffects[effect]
        if curr[0] == 1: del fit.projEffects[effect]
        else: fit.projEffects[effect] = (curr[0] -1, curr[1], curr[2])
        self.emit("fitting-changed")
    elif event.button == 3:
        fit = self.getActiveFit()
        if fit == None: return
        self.projectedAddFitEntry.set_submenu(self.generateFitMenu(self.addProjFit))
        effect = self.getSelectedProjEffect()
        children = self.projectedMenu.get_children()
        if effect and not self.projectedViewStatsEntry in children:
            self.projectedMenu.append(self.projectedViewStatsEntry)
        elif not effect and self.projectedViewStatsEntry in children:
            self.projectedMenu.remove(self.projectedViewStatsEntry)
            
        if effect and not isinstance(effect, fitting.fitting)\
         and not self.projectedViewGroupEntry in children:
            self.projectedMenu.append(self.projectedViewGroupEntry)
        elif (not effect or isinstance(effect, fitting.fitting))\
         and  self.projectedViewGroupEntry in children:
            self.projectedMenu.remove(self.projectedViewGroupEntry)
            
        self.projectedMenu.popup(None, None, None, event.button, event.time)
        self.projectedMenu.show_all()

def addProjEffect(self, fit, effect):
    if effect in fit.projEffects:
        curr = fit.projEffects[effect]
        fit.projEffects[effect] = (curr[0] + 1, curr[1], curr[2])
    else:
        if isinstance(effect, item.item): overrides = {"state" : fitting.STATE_ACTIVE}
        else: overrides = {}
        fit.projEffects[effect] = (1, 0, overrides)
    
def addProjModule(self, *stuff):
    activeFit = self.getActiveFit()
    if activeFit == None: return
    mod = self.getSelectedItem()
    self.addProjEffect(activeFit, mod)
    self.emit("fitting-changed")
    
def addProjFit(self, widget, fit):
    activeFit = self.getActiveFit()
    if activeFit == None: return
    self.addProjEffect(activeFit, fit)
    self.emit("fitting-changed")
